"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.connect = connect;
exports["default"] = responder;

var _reselect = require("reselect");

function _toConsumableArray(arr) { return _arrayWithoutHoles(arr) || _iterableToArray(arr) || _nonIterableSpread(); }

function _nonIterableSpread() { throw new TypeError("Invalid attempt to spread non-iterable instance"); }

function _iterableToArray(iter) { if (Symbol.iterator in Object(iter) || Object.prototype.toString.call(iter) === "[object Arguments]") return Array.from(iter); }

function _arrayWithoutHoles(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = new Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } }

function isPromise(n) {
  return n !== null && typeof n.then === 'function';
}

function getDispatcher(actionCreator, dispatch) {
  return function (value) {
    var action = actionCreator(value);
    return dispatch(action);
  };
}

function doPromise(promise, successActionCreator, failureActionCreator, dispatch) {
  return isPromise(promise) ? promise.then(getDispatcher(successActionCreator, dispatch))["catch"](getDispatcher(failureActionCreator, dispatch)) : Promise.resolve();
}

function connect(responders, store) {
  var dispatch = store.dispatch;
  return responders.map(function (r) {
    var listener = function listener() {
      var state = store.getState();
      r(state, dispatch);
    };

    return store.subscribe(listener);
  });
}

function responder(selectors, promiseProducer, successActionCreator, failureActionCreator) {
  var selector = _reselect.createSelector.apply(void 0, _toConsumableArray(selectors).concat([function () {
    for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    return args;
  }]));

  var prevRecomputations = 0;

  var r = function r(state, dispatch) {
    var values = selector(state);
    var currentRecomputations = selector.recomputations();

    if (currentRecomputations !== prevRecomputations) {
      var promise = promiseProducer.apply(void 0, _toConsumableArray(values));
      var result = doPromise(promise, successActionCreator, failureActionCreator, dispatch);
      prevRecomputations = currentRecomputations;
      return result;
    }

    return Promise.resolve();
  };

  r.recomputations = function () {
    return selector.recomputations();
  };

  r.resetRecomputations = function () {
    return selector.resetRecomputations();
  };

  return r;
}